"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericProviderSDK = void 0;
const Utils_1 = __importDefault(require("../../Utils"));
const RegionsService_1 = require("../../regions/RegionsService");
var QuoteType;
(function (QuoteType) {
    QuoteType["buy"] = "BUY";
    QuoteType["sell"] = "SELL";
})(QuoteType || (QuoteType = {}));
class GenericProviderSDK {
    constructor(onRampSdk, providerId, logger) {
        this.onRampSdk = onRampSdk;
        this.providerId = providerId;
        this.logger = logger;
    }
    getId() {
        return this.providerId;
    }
    async getLimit(provider, fiat, crypto, paymentMethods) {
        var _a, _b, _c, _d, _e, _f;
        let minAmount = Infinity;
        let maxAmount = -Infinity;
        for (const paymentMethod of paymentMethods) {
            const fiatLimit = (_b = (_a = provider.supportedFiatCurrencies.find((f) => f.id == fiat)) === null || _a === void 0 ? void 0 : _a.limits) === null || _b === void 0 ? void 0 : _b[paymentMethod];
            let cryptoLimit = (_e = (_d = (_c = provider.supportedCryptoCurrencies
                .find((c) => c.id === crypto)) === null || _c === void 0 ? void 0 : _c.limits) === null || _d === void 0 ? void 0 : _d.payments) === null || _e === void 0 ? void 0 : _e.find((paymentLimit) => paymentLimit.payment === paymentMethod);
            if (!cryptoLimit) {
                cryptoLimit = (_f = provider.supportedCryptoCurrencies.find((c) => c.id === crypto)) === null || _f === void 0 ? void 0 : _f.limits;
            }
            const methodMinAmount = (() => {
                if ((fiatLimit === null || fiatLimit === void 0 ? void 0 : fiatLimit.minAmount) && (cryptoLimit === null || cryptoLimit === void 0 ? void 0 : cryptoLimit.minAmount)) {
                    return Math.max(fiatLimit.minAmount, cryptoLimit.minAmount);
                }
                else if (fiatLimit === null || fiatLimit === void 0 ? void 0 : fiatLimit.minAmount) {
                    return fiatLimit.minAmount;
                }
                else if (cryptoLimit === null || cryptoLimit === void 0 ? void 0 : cryptoLimit.minAmount) {
                    return cryptoLimit.minAmount;
                }
                else {
                    return -Infinity;
                }
            })();
            const methodMaxAmount = (() => {
                if ((fiatLimit === null || fiatLimit === void 0 ? void 0 : fiatLimit.maxAmount) && (cryptoLimit === null || cryptoLimit === void 0 ? void 0 : cryptoLimit.maxAmount)) {
                    return Math.min(fiatLimit.maxAmount, cryptoLimit.maxAmount);
                }
                else if (fiatLimit === null || fiatLimit === void 0 ? void 0 : fiatLimit.maxAmount) {
                    return fiatLimit.maxAmount;
                }
                else if (cryptoLimit === null || cryptoLimit === void 0 ? void 0 : cryptoLimit.maxAmount) {
                    return cryptoLimit.maxAmount;
                }
                else {
                    return Infinity;
                }
            })();
            minAmount = Math.min(minAmount, methodMinAmount);
            maxAmount = Math.max(maxAmount, methodMaxAmount);
        }
        if (minAmount === Infinity)
            minAmount = -Infinity;
        if (maxAmount === -Infinity)
            maxAmount = Infinity;
        return {
            minAmount,
            maxAmount,
            feeDynamicRate: 0,
            feeFixedRate: 0,
            quickAmounts: [],
        };
    }
    async getSellLimit(provider, fiat, crypto, paymentMethods) {
        var _a, _b, _c, _d, _e;
        let minAmount = Infinity;
        let maxAmount = -Infinity;
        const cryptoCurrency = provider.supportedCryptoCurrencies.find((c) => c.id === crypto);
        for (const paymentMethod of paymentMethods) {
            const sellLimits = ((_b = (_a = cryptoCurrency === null || cryptoCurrency === void 0 ? void 0 : cryptoCurrency.sell) === null || _a === void 0 ? void 0 : _a.limits) === null || _b === void 0 ? void 0 : _b[paymentMethod]) ||
                ((_c = cryptoCurrency === null || cryptoCurrency === void 0 ? void 0 : cryptoCurrency.sell) === null || _c === void 0 ? void 0 : _c.limits);
            if (sellLimits) {
                minAmount = Math.min(minAmount, (_d = sellLimits.minAmount) !== null && _d !== void 0 ? _d : -Infinity);
                maxAmount = Math.max(maxAmount, (_e = sellLimits.maxAmount) !== null && _e !== void 0 ? _e : Infinity);
            }
        }
        if (minAmount === Infinity)
            minAmount = -Infinity;
        if (maxAmount === -Infinity)
            maxAmount = Infinity;
        return {
            minAmount,
            maxAmount,
            feeDynamicRate: 0,
            feeFixedRate: 0,
            quickAmounts: [],
        };
    }
    async addApplePayToQuote(provider, regionId, quote, merchantId) {
        const methodData = [
            {
                supportedMethods: ['apple-pay'],
                data: {
                    countryCode: this.getCountryCode(regionId),
                    currencyCode: this.translateFiat(provider, quote.fiatId),
                    supportedNetworks: ['visa', 'mastercard', 'discover'],
                    merchantIdentifier: merchantId,
                },
            },
        ];
        const translatedFiat = this.translateFiat(provider, quote.fiatId);
        const paymentDetails = (setup) => {
            return {
                displayItems: [
                    {
                        amount: {
                            currency: translatedFiat,
                            value: `${(quote.amountIn -
                                quote.providerFee -
                                quote.networkFee).toFixed(quote.fiat.decimals)}`,
                        },
                        label: setup.getPurchaseFiatAmountWithoutFeeLabel(quote.crypto),
                    },
                    {
                        amount: {
                            currency: translatedFiat,
                            value: `${(quote.networkFee + quote.providerFee).toFixed(quote.fiat.decimals)}`,
                        },
                        label: setup.getPurchaseFiatFeeLabel(),
                    },
                ],
                total: {
                    amount: {
                        translatedFiat,
                        value: `${quote.amountIn.toFixed(quote.fiat.decimals)}`,
                    },
                    label: setup.getPurchaseFiatTotalAmountLabel(),
                },
            };
        };
        const paymentOptions = {
            requestPayerPhone: true,
            requestPayerEmail: true,
            requestPayerName: true,
            requestBilling: true,
            merchantCapabilities: ['debit', 'credit'],
        };
        quote.getApplePayRequestInfo = (setup) => {
            return {
                methodData: methodData,
                paymentDetails: paymentDetails(setup),
                paymentOptions: paymentOptions,
            };
        };
        quote.purchaseWithApplePay = async (paymentDetails) => {
            const { billingContact: { postalAddress, name }, paymentData, paymentMethod, shippingContact, transactionIdentifier, } = paymentDetails;
            const formattedBillingContact = {
                addressLines: postalAddress.street.split('\n'),
                administrativeArea: postalAddress.state,
                country: postalAddress.country,
                countryCode: postalAddress.ISOCountryCode,
                familyName: name.familyName,
                givenName: name.givenName,
                locality: postalAddress.city,
                postalCode: postalAddress.postalCode,
                subAdministrativeArea: postalAddress.subAdministrativeArea,
                subLocality: postalAddress.subLocality,
            };
            const formattedShippingContact = Object.assign(Object.assign(Object.assign({}, formattedBillingContact), (paymentOptions.requestPayerEmail
                ? { emailAddress: shippingContact.emailAddress }
                : {})), (paymentOptions.requestPayerPhone
                ? { phoneNumber: shippingContact.phoneNumber }
                : {}));
            const payload = {
                paymentObject: {
                    billingContact: formattedBillingContact,
                    shippingContact: formattedShippingContact,
                    token: {
                        paymentData,
                        paymentMethod,
                        transactionIdentifier,
                    },
                },
                order: {
                    region: regionId,
                    country: this.getCountryCode(regionId),
                    crypto: quote.cryptoTranslation,
                    fiat: this.translateFiat(provider, quote.fiatId),
                    payment: 'apple-pay',
                    amount: `${quote.amountIn}`,
                    walletAddress: quote.receiver,
                    orderCustomId: null,
                    partnerAccountId: null,
                    redirectUrl: this.onRampSdk.sdkParameters.environment.orders,
                },
            };
            const ordersService = await this.onRampSdk.orders();
            const applePayResult = await ordersService.submitApplePayOrder(quote.receiver, provider === null || provider === void 0 ? void 0 : provider.id, payload);
            return applePayResult;
        };
    }
    async formatQuote(quoteOrQuery, provider, regionId, paymentMethod, crypto, fiat, amount, action, receiver) {
        const quote = quoteOrQuery['quote'];
        const metadata = quoteOrQuery['metadata'];
        const nativeApplePay = quoteOrQuery['nativeApplePay'];
        const quoteFormatted = {
            provider: provider,
            crypto: quote.crypto,
            cryptoTranslation: quote.cryptoTranslation,
            cryptoId: crypto,
            fiat: quote.fiat,
            fiatId: fiat,
            networkFee: quote.networkFee,
            providerFee: quote.providerFee,
            extraFee: quote.extraFee,
            amountIn: Number(quote.amountIn),
            amountOut: quote.amountOut,
            paymentMethod: quote.paymentMethod,
            receiver: quote.receiver,
            isNativeApplePay: nativeApplePay && (nativeApplePay === null || nativeApplePay === void 0 ? void 0 : nativeApplePay.supported),
            isCustomAction: quote === null || quote === void 0 ? void 0 : quote.isCustomAction,
            tags: metadata.tags,
        };
        if (action === RegionsService_1.ProviderAction.buy) {
            ;
            quoteFormatted.buy = receiver
                ? async () => {
                    const regions = await this.onRampSdk.regions();
                    const buyAction = await regions.getBuyUrl(provider, regionId, paymentMethod, crypto, fiat, amount, receiver);
                    return buyAction;
                }
                : () => {
                    throw new Error('Quote is readonly');
                };
            if (nativeApplePay) {
                await this.addApplePayToQuote(provider, regionId, quoteFormatted, nativeApplePay.merchantId);
            }
        }
        else if (action === RegionsService_1.ProviderAction.sell) {
            ;
            quoteFormatted.sell = receiver
                ? async () => {
                    const regions = await this.onRampSdk.regions();
                    const sellAction = await regions.getSellUrl(provider, regionId, paymentMethod, crypto, fiat, amount, receiver);
                    return sellAction;
                }
                : () => {
                    throw new Error('Quote is readonly');
                };
        }
        return quoteFormatted;
    }
    async getQuote(provider, quoteUri, regionId, paymentMethod, crypto, fiat, amount, receiver, abortController) {
        const quoteOrQuery = await this.getRemoteQuote(provider, regionId, paymentMethod, crypto, fiat, amount, receiver, abortController);
        if (quoteOrQuery['error']) {
            return quoteOrQuery;
        }
        else if (!quoteOrQuery['quote']) {
            return {
                error: true,
                status: null,
                message: 'Unable to get quote from ' + this.providerId,
                technicalDetails: 'Quote object is empty',
                provider: provider.provider,
                canRetry: false,
            };
        }
        const quoteFormatted = (await this.formatQuote(quoteOrQuery, provider.provider, regionId, paymentMethod, crypto, fiat, amount, RegionsService_1.ProviderAction.buy, receiver));
        return quoteFormatted;
    }
    async getRemoteQuote(provider, regionId, paymentMethod, crypto, fiat, amount, receiver, abortController, type) {
        let baseUrl = this.onRampSdk.sdkParameters.environment.orders;
        if (baseUrl.endsWith('/')) {
            baseUrl = baseUrl.substring(0, baseUrl.length - 1);
        }
        if (type === QuoteType.sell) {
            baseUrl += `${provider.provider.id}/sell-quote`;
        }
        else {
            baseUrl += `${provider.provider.id}/quote`;
        }
        const url = new URL(baseUrl);
        url.searchParams.set('regionId', regionId);
        url.searchParams.set('paymentMethodId', paymentMethod);
        url.searchParams.set('cryptoCurrencyId', crypto);
        url.searchParams.set('fiatCurrencyId', fiat);
        url.searchParams.set('amount', amount.toString());
        if (receiver) {
            url.searchParams.set('walletAddress', receiver);
        }
        const headers = {};
        const response = await Utils_1.default.try(async () => await this.onRampSdk.getQuote({
            validateStatus: function () {
                return true;
            },
            headers: headers,
            url: url.toString(),
            method: 'GET',
            signal: abortController === null || abortController === void 0 ? void 0 : abortController.signal,
        }));
        if (response.status != 200) {
            return this.parseError(response);
        }
        const query = response.data;
        return query;
    }
    async getSellQuote(provider, quoteUri, regionId, paymentMethod, crypto, fiat, amount, receiver, abortController) {
        const quoteOrQuery = await this.getRemoteQuote(provider, regionId, paymentMethod, crypto, fiat, amount, receiver, abortController, QuoteType.sell);
        if (quoteOrQuery['error']) {
            return quoteOrQuery;
        }
        else if (!quoteOrQuery['quote']) {
            return {
                error: true,
                status: null,
                message: 'Unable to get quote from ' + this.providerId,
                technicalDetails: 'Quote object is empty',
                provider: provider.provider,
                canRetry: false,
            };
        }
        const quote = quoteOrQuery['quote'];
        const metadata = quoteOrQuery['metadata'];
        const quoteFormatted = {
            provider: provider.provider,
            crypto: quote.crypto,
            cryptoId: crypto,
            fiat: quote.fiat,
            fiatId: fiat,
            networkFee: quote.networkFee,
            providerFee: quote.providerFee,
            extraFee: quote.extraFee,
            amountIn: quote.amountIn,
            amountOut: quote.amountOut,
            sell: receiver
                ? async () => {
                    const regions = await this.onRampSdk.regions();
                    const buyAction = await regions.getSellUrl(provider.provider, regionId, paymentMethod, crypto, fiat, amount, receiver);
                    return buyAction;
                }
                : () => {
                    throw new Error('Quote is readonly');
                },
            paymentMethod: quote.paymentMethod,
            receiver: quote.receiver,
            isCustomAction: quote === null || quote === void 0 ? void 0 : quote.isCustomAction,
            tags: metadata.tags,
        };
        return quoteFormatted;
    }
    getCountryCode(regionId) {
        const parts = regionId.split('/');
        const regionCode = parts[parts.length - 1];
        const countryCode = regionCode.split('-')[0];
        return countryCode.toUpperCase();
    }
    parseError(body) {
        return {
            provider: undefined,
            error: true,
            status: body.error.statusCode,
            message: body.error.message,
            canRetry: false,
        };
    }
    translateFiat(provider, currencyId) {
        const split = currencyId.split('/');
        return split[split.length - 1].toUpperCase();
    }
    translateCrypto(provider, currencyId) {
        return provider.supportedCryptoCurrencies.filter((c) => c.id === currencyId)[0].translation;
    }
}
exports.GenericProviderSDK = GenericProviderSDK;
