"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const RegionsService_1 = __importDefault(require("./RegionsService"));
const Utils_1 = __importDefault(require("../Utils"));
const CountryCache_1 = require("./CountryCache");
describe('RegionsService', () => {
    let consoleSpy;
    let axiosMock;
    let loggerMock;
    let sdkMock;
    let sdkParametersMock;
    beforeEach(() => {
        consoleSpy = jest.spyOn(console, 'warn').mockImplementation();
        axiosMock = {
            get: jest.fn().mockResolvedValue({ data: [] }),
        };
        loggerMock = { log: jest.fn() };
        sdkMock = {};
        sdkParametersMock = {
            environment: {
                regions: 'http://example.com',
                orders: 'http://example.com/orders',
            },
        };
        jest.spyOn(Utils_1.default, 'downloadAsMap').mockResolvedValue(new Map());
        RegionsService_1.default['instanceCount'] = 0;
    });
    afterEach(() => {
        consoleSpy.mockRestore();
        jest.restoreAllMocks();
    });
    describe('create method', () => {
        it('should create an instance without warning when it is the first one', async () => {
            const service = await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            expect(service).toBeInstanceOf(RegionsService_1.default);
            expect(consoleSpy).not.toHaveBeenCalled();
        });
        it('should show warning when creating a second instance', async () => {
            await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            expect(consoleSpy).toHaveBeenCalledWith('1 other instance of the regions service already exists. The regions service is likely being used by multiple SDK instance. This might result in increased network activity and poor performance. If possible, OnRampSdk.getSdk().regions() should only be called once in your application or per webpage, as it contains cached data.');
        });
        it('should show plural warning when creating multiple instances', async () => {
            await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            expect(consoleSpy).toHaveBeenLastCalledWith('2 other instances of the regions service already exist. The regions service is likely being used by multiple SDK instance. This might result in increased network activity and poor performance. If possible, OnRampSdk.getSdk().regions() should only be called once in your application or per webpage, as it contains cached data.');
        });
        it('should initialize the instance correctly', async () => {
            const initSpy = jest.spyOn(RegionsService_1.default.prototype, 'init');
            const service = await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
            expect(service).toBeInstanceOf(RegionsService_1.default);
            expect(initSpy).toHaveBeenCalledWith(sdkMock);
            initSpy.mockRestore();
        });
    });
    describe('service methods', () => {
        let service;
        beforeEach(async () => {
            service = await RegionsService_1.default.create(axiosMock, loggerMock, sdkMock, sdkParametersMock);
        });
        describe('getCountries', () => {
            it('should return cached countries if available', async () => {
                const mockCountries = new Map([
                    ['country1', { id: 'country1', name: 'Country 1' }],
                    ['country2', { id: 'country2', name: 'Country 2' }],
                ]);
                jest.spyOn(Utils_1.default, 'downloadAsMap').mockResolvedValueOnce(mockCountries);
                await service.reset(sdkMock);
                const countries = await service.getCountries();
                expect(countries).toHaveLength(2);
                expect(countries[0].name).toBe('Country 1');
            });
            it('should download countries if cache is empty', async () => {
                const mockCountries = new Map([
                    ['country1', { id: 'country1', name: 'Country 1' }],
                ]);
                jest.spyOn(Utils_1.default, 'downloadAsMap').mockResolvedValueOnce(mockCountries);
                const countries = await service.getCountries();
                expect(countries).toHaveLength(1);
                expect(countries[0].name).toBe('Country 1');
            });
        });
        describe('getPaymentMethods', () => {
            it.skip('should return payment methods for a region', async () => {
                const mockCache = {
                    cachedPayments: [
                        { id: 'payment1', buy: { enabled: true } },
                        { id: 'payment2', buy: { enabled: false } },
                    ],
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const payments = await service.getPaymentMethods('region1');
                expect(payments).toHaveLength(1);
                expect(payments[0].id).toBe('payment1');
            });
            it('should throw error if regionId is missing', async () => {
                await expect(service.getPaymentMethods(undefined)).rejects.toThrow();
            });
        });
        describe('getFiatCurrencies', () => {
            it.skip('should return fiat currencies for a region and payment method', async () => {
                const mockCache = {
                    cachedFiatCurrencies: new Map([
                        [
                            'fiat1',
                            { fiatCurrency: { id: 'fiat1' }, excludedPaymentMethods: [] },
                        ],
                        [
                            'fiat2',
                            {
                                fiatCurrency: { id: 'fiat2' },
                                excludedPaymentMethods: ['payment1'],
                            },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const fiats = await service.getFiatCurrencies('region1', ['payment1']);
                expect(fiats).toHaveLength(1);
                expect(fiats[0].id).toBe('fiat1');
            });
        });
        describe('getCryptoCurrencies', () => {
            it.skip('should return crypto currencies for given parameters', async () => {
                const mockCache = {
                    cachedPayments: [{ id: 'payment1' }],
                    cachedFiatCurrencies: new Map([['fiat1', {}]]),
                    cachedCryptoCurrencies: new Map([
                        [
                            'crypto1',
                            {
                                cryptoCurrency: { id: 'crypto1' },
                                excludedPaymentMethods: [],
                                excludedFiat: [],
                                excludedFiatAndPaymentMethods: [],
                            },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const cryptos = await service.getCryptoCurrencies('region1', ['payment1'], 'fiat1');
                expect(cryptos).toHaveLength(1);
                expect(cryptos[0].id).toBe('crypto1');
            });
        });
        describe('getParameters', () => {
            it('should return region parameters', async () => {
                const mockParameters = {
                    disableLimits: false,
                    defaultOrderCustomIdSeparator: '-',
                };
                const mockCache = {
                    cachedParameters: mockParameters,
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const parameters = await service.getParameters('region1');
                expect(parameters).toEqual(mockParameters);
            });
        });
        describe('generateCustomOrderId', () => {
            beforeEach(() => {
                const mockCache = {
                    cachedParameters: {
                        defaultOrderCustomIdSeparator: '-',
                    },
                    cachedProviders: new Map(),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
            });
            it('should generate custom order id with expiration', async () => {
                const provider = {
                    id: 'provider1',
                    features: {
                        buy: {
                            orderCustomId: 'Digits10',
                            orderCustomIdExpiration: 60,
                        },
                    },
                };
                const orderId = await service.generateCustomOrderId(provider, 'region1');
                expect(orderId).toMatch(/provider1\/orders\/\d{10}-\d+/);
            });
            it('should generate custom order id without expiration', async () => {
                const provider = {
                    id: 'provider1',
                    features: {
                        buy: {
                            orderCustomId: 'Digits10',
                        },
                    },
                };
                const orderId = await service.generateCustomOrderId(provider, 'region1');
                expect(orderId).toMatch(/provider1\/orders\/\d{10}/);
            });
        });
        describe('getSellCountries', () => {
            it('should return only countries with sell support', async () => {
                const mockCountries = new Map([
                    [
                        'country1',
                        {
                            id: 'country1',
                            name: 'Country 1',
                            support: { sell: true },
                            states: [
                                { id: 'state1', support: { sell: true } },
                                { id: 'state2', support: { sell: false } },
                            ],
                        },
                    ],
                    [
                        'country2',
                        {
                            id: 'country2',
                            name: 'Country 2',
                            support: { sell: false },
                        },
                    ],
                ]);
                jest.spyOn(Utils_1.default, 'downloadAsMap').mockResolvedValueOnce(mockCountries);
                await service.reset(sdkMock);
                const countries = await service.getSellCountries();
                expect(countries).toHaveLength(1);
                expect(countries[0].states).toHaveLength(1);
                expect(countries[0].states[0].id).toBe('state1');
            });
        });
        describe('getSellPaymentMethods', () => {
            it.skip('should return payment methods with sell enabled', async () => {
                const mockCache = {
                    cachedPayments: [
                        { id: 'payment1', sell: { enabled: true } },
                        { id: 'payment2', sell: { enabled: false } },
                    ],
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const payments = await service.getSellPaymentMethods('region1');
                expect(payments).toHaveLength(1);
                expect(payments[0].id).toBe('payment1');
            });
        });
        describe('getSellFiatCurrencies', () => {
            it.skip('should return fiat currencies for sell', async () => {
                const mockCache = {
                    cachedFiatCurrencies: new Map([
                        [
                            'fiat1',
                            {
                                fiatCurrency: { id: 'fiat1' },
                                sell: {
                                    enabled: true,
                                    excludedPaymentMethods: [],
                                },
                            },
                        ],
                        [
                            'fiat2',
                            {
                                fiatCurrency: { id: 'fiat2' },
                                sell: {
                                    enabled: true,
                                    excludedPaymentMethods: ['payment1'],
                                },
                            },
                        ],
                        [
                            'fiat3',
                            {
                                fiatCurrency: { id: 'fiat3' },
                                sell: { enabled: false },
                            },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const fiats = await service.getSellFiatCurrencies('region1', [
                    'payment1',
                ]);
                expect(fiats).toHaveLength(1);
                expect(fiats[0].id).toBe('fiat1');
            });
        });
        describe('getSellCryptoCurrencies', () => {
            it.skip('should return crypto currencies for sell', async () => {
                const mockCache = {
                    cachedPayments: [{ id: 'payment1' }],
                    cachedFiatCurrencies: new Map([['fiat1', {}]]),
                    cachedCryptoCurrencies: new Map([
                        [
                            'crypto1',
                            {
                                cryptoCurrency: { id: 'crypto1' },
                                sell: {
                                    enabled: true,
                                    excludedPaymentMethods: [],
                                    excludedFiat: [],
                                    excludedFiatAndPaymentMethods: [],
                                },
                            },
                        ],
                        [
                            'crypto2',
                            {
                                cryptoCurrency: { id: 'crypto2' },
                                sell: { enabled: false },
                            },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const cryptos = await service.getSellCryptoCurrencies('region1', ['payment1'], 'fiat1');
                expect(cryptos).toHaveLength(1);
                expect(cryptos[0].id).toBe('crypto1');
            });
        });
        describe('getAllFiatCurrencies', () => {
            it.skip('should return all fiat currencies for a region', async () => {
                const mockCache = {
                    cachedFiatCurrencies: new Map([
                        ['fiat1', { fiatCurrency: { id: 'fiat1', name: 'Fiat 1' } }],
                        ['fiat2', { fiatCurrency: { id: 'fiat2', name: 'Fiat 2' } }],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const fiats = await service.getAllFiatCurrencies('region1');
                expect(fiats).toHaveLength(2);
                expect(fiats[0].id).toBe('fiat1');
            });
        });
        describe('getAllCryptoCurrencies', () => {
            it.skip('should return all crypto currencies for a region', async () => {
                const mockCache = {
                    cachedCryptoCurrencies: new Map([
                        [
                            'crypto1',
                            { cryptoCurrency: { id: 'crypto1', name: 'Crypto 1' } },
                        ],
                        [
                            'crypto2',
                            { cryptoCurrency: { id: 'crypto2', name: 'Crypto 2' } },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const cryptos = await service.getAllCryptoCurrencies('region1');
                expect(cryptos).toHaveLength(2);
                expect(cryptos[0].id).toBe('crypto1');
            });
        });
        describe('getDefaultFiatCurrency', () => {
            it.skip('should return default fiat currency based on country settings', async () => {
                const mockCountries = [
                    {
                        id: 'region1',
                        currencies: ['/currencies/fiat/eur', '/currencies/fiat/usd'],
                    },
                ];
                const mockFiats = [
                    {
                        fiatCurrency: { id: '/currencies/fiat/eur' },
                        excludedPaymentMethods: [],
                    },
                ];
                const result = service.getDefaultFiatCurrencySync(mockCountries, mockFiats, 'region1', ['payment1']);
                expect(result.id).toBe('/currencies/fiat/eur');
            });
            it.skip('should fallback to USD if local currency not supported', async () => {
                const mockCountries = [
                    {
                        id: 'region1',
                        currencies: ['/currencies/fiat/local'],
                    },
                ];
                const mockFiats = [
                    {
                        fiatCurrency: { id: '/currencies/fiat/usd' },
                        excludedPaymentMethods: [],
                    },
                ];
                const result = service.getDefaultFiatCurrencySync(mockCountries, mockFiats, 'region1', ['payment1']);
                expect(result.id).toBe('/currencies/fiat/usd');
            });
        });
        describe('getCountryCache', () => {
            it.skip('should create new cache when not exists', async () => {
                const mockCache = {
                    expirationDate: new Date(Date.now() + 1000000),
                    cachedParameters: {},
                };
                const createSpy = jest
                    .spyOn(CountryCache_1.CountryCache, 'Create')
                    .mockResolvedValue(mockCache);
                await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                expect(createSpy).toHaveBeenCalledWith(expect.anything(), expect.anything(), expect.anything(), 'region1', undefined);
            });
            it('should reuse existing cache when not expired', async () => {
                const mockCache = {
                    expirationDate: new Date(Date.now() + 1000000),
                    cachedParameters: {},
                };
                const createSpy = jest
                    .spyOn(CountryCache_1.CountryCache, 'Create')
                    .mockResolvedValue(mockCache);
                await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                expect(createSpy).toHaveBeenCalledTimes(1);
            });
            it('should refresh expired cache', async () => {
                const expiredCache = {
                    expirationDate: new Date(Date.now() - 1000000),
                    cachedParameters: {},
                };
                const newCache = {
                    expirationDate: new Date(Date.now() + 1000000),
                    cachedParameters: {},
                };
                const createSpy = jest
                    .spyOn(CountryCache_1.CountryCache, 'Create')
                    .mockResolvedValueOnce(expiredCache)
                    .mockResolvedValueOnce(newCache);
                await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                expect(createSpy).toHaveBeenCalledTimes(2);
            });
            it('should keep using expired cache if refresh fails', async () => {
                const expiredCache = {
                    expirationDate: new Date(Date.now() - 1000000),
                    cachedParameters: {},
                };
                const createSpy = jest
                    .spyOn(CountryCache_1.CountryCache, 'Create')
                    .mockResolvedValue(expiredCache);
                createSpy.mockClear();
                const cache1 = await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                createSpy.mockRejectedValueOnce(new Error('Network error'));
                const cache2 = await service['getCountryCache'](null, 'region1', [], null, null, new AbortController());
                expect(cache1).toBe(cache2);
                expect(createSpy).toHaveBeenCalledTimes(3);
            });
        });
        describe('getLimits', () => {
            it('should return disabled limits when configured', async () => {
                const mockCache = {
                    cachedParameters: {
                        disableLimits: true,
                    },
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const limits = await service.getLimits('region1', ['payment1'], 'crypto1', 'fiat1');
                expect(limits.minAmount).toBe(-Infinity);
                expect(limits.maxAmount).toBe(Infinity);
            });
            it.skip('should aggregate limits from all providers', async () => {
                const mockCache = {
                    cachedParameters: {
                        disableLimits: false,
                    },
                    cachedProviders: new Map([
                        [
                            'provider1',
                            {
                                provider: { id: 'provider1' },
                                supportedPaymentMethods: [{ id: 'payment1' }],
                                supportedCryptoCurrencies: [
                                    {
                                        id: 'crypto1',
                                        excludedFiat: [],
                                        excludedFiatAndPaymentMethods: [],
                                    },
                                ],
                                supportedFiatCurrencies: [
                                    {
                                        id: 'fiat1',
                                        excludedPaymentMethods: [],
                                    },
                                ],
                            },
                        ],
                    ]),
                };
                jest.spyOn(CountryCache_1.CountryCache, 'Create').mockResolvedValue(mockCache);
                const mockProviderSdk = {
                    getLimit: jest.fn().mockResolvedValue({
                        minAmount: 100,
                        maxAmount: 1000,
                        feeDynamicRate: 0.01,
                        feeFixedRate: 1,
                    }),
                };
                service['providers'] = {
                    get: jest.fn().mockReturnValue(mockProviderSdk),
                };
                const limits = await service.getLimits('region1', ['payment1'], 'crypto1', 'fiat1');
                expect(limits.minAmount).toBe(100);
                expect(limits.maxAmount).toBe(1000);
                expect(mockProviderSdk.getLimit).toHaveBeenCalled();
            });
        });
        describe('getNetworkDetails', () => {
            it('should return network details', async () => {
                const mockNetworkDetails = [{ id: 'network1', name: 'Network 1' }];
                axiosMock.get.mockResolvedValueOnce({ data: mockNetworkDetails });
                const details = await service.getNetworkDetails();
                expect(details).toEqual(mockNetworkDetails);
                expect(axiosMock.get).toHaveBeenCalledWith('/regions/network-details');
            });
        });
        describe('reset', () => {
            it('should clear caches and reinitialize', async () => {
                const initSpy = jest.spyOn(service, 'init');
                await service.reset(sdkMock);
                expect(initSpy).toHaveBeenCalledWith(sdkMock);
            });
        });
    });
});
